/*
 * $Source: /cvs/sources/com/odisei/sdk/samples/addtoaddress/AddToAddressObserver.java,v $
 * $Revision: 1.7 $
 * $Date: 1999/09/30 14:49:32 $
 * $Author: kris $
 *
 * Copyright (c) 1999 by 8x8, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of 8x8. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with 8x8.
 *
 */

import java.io.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;


/**
* This class observes and displays CallControl events.
* When it detects a Connection Initiated event (as a result of a hookswitch-off
* by the user), it prompts the user to hit return after which
* an addToAddress() is invoked to establish a connnection to the 102 terminal.
*
* Additional information about executed JTAPI services is displayed when the
* behaviour is not defined unambiguosly by Suns official JTAPI documentation.
*/
public class AddToAddressObserver implements CallControlCallObserver {
	private Terminal myTerminal = null;
	private Thread thread = null;

	public AddToAddressObserver(Terminal terminal) {
		myTerminal = terminal;
		}

	/** Implementation of callChangedEvent */
	public synchronized void callChangedEvent(CallEv[] eventList) {
 		CallCtlConnEv connEvent = null;
 		CallCtlTermConnEv termConnEvent = null;
 		String msg = null;

 		for (int i = 0; i < eventList.length; i++) {

 			if (eventList[i] instanceof CallCtlConnEv) {
 				connEvent = (CallCtlConnEv)eventList[i];
 				Address address = connEvent.getConnection().getAddress();
 				msg = "Connection to address : " + address.getName() + " is ";
			}

 			if (eventList[i] instanceof CallCtlTermConnEv) {
 				termConnEvent = (CallCtlTermConnEv)eventList[i];
 				Terminal terminal = termConnEvent.getTerminalConnection().getTerminal();
 				msg = "TerminalConnection : " + terminal.getName() + " is ";
			}

 			switch(eventList[i].getID()) {
	 			case CallCtlConnFailedEv.ID :
	 				System.out.println(msg + "Failed");
	 				break;

	 			case CallCtlConnAlertingEv.ID :
	 				System.out.println(msg + "Alerting");
	 				break;

 				case CallCtlConnInitiatedEv.ID :
 					System.out.println(msg + "Initiated");
					final CallControlConnection connection = (CallControlConnection)connEvent.getConnection();
					if (thread != null)	thread.interrupt();  
					
					thread = new Thread() {
						public void run(){
						
							switch(AddToAddressMain.executeRequest("Hit <enter> to execute addToAddress(102) service")) {
								case AddToAddressMain.SHUTDOWN :
	 								AddToAddressMain.shutdown();									
									break;
									
								case AddToAddressMain.ACTION :
			 						try {	
					                  // It is a good attitude not to block JTAPI event handlers
 										connection.addToAddress("1");
 										connection.addToAddress("02");
            		    	           // We could also have said 'connection.addToAddress("102")' directly.
                		    	       // AddToAddress()'s purpose is to allow sending digit by digit.
 									}
 									catch (Exception e) {
 										System.out.println("can't invoke addToAddress" + e.toString());
									}
 									break;
 									
 								case AddToAddressMain.NO_ACTION :
 									break;
									 	
							}
						}

 						public void interrupt() {
 							AddToAddressMain.stopRequest();
 							super.interrupt();	
						}
					} ;
 					thread.start();	
	 				break;

 				case CallCtlConnDialingEv.ID :
 					System.out.println(msg + "Dialing");
 					break;

 				case  CallCtlConnDisconnectedEv.ID :
 					System.out.println(msg + "Disconnected");
 					break;

 				case CallCtlConnEstablishedEv.ID :
 					System.out.println(msg + "Established");
 					break;

 				case CallCtlTermConnDroppedEv.ID :
 					System.out.println(msg + "Dropped");
 					break;

 				case CallCtlTermConnRingingEv.ID :
	 				System.out.println(msg + "Alerting, please pick-up this phone.");
	 				break;

				case CallCtlTermConnTalkingEv.ID :
					System.out.println(msg + "Talking");

					Terminal terminal = termConnEvent.getTerminalConnection().getTerminal();
					if (myTerminal.equals(terminal)) break;

					Runnable runnable = new Runnable() {
						public void run() {
							try {
								Thread.sleep(6000);
								System.out.println("Please hang-up a phone :");
								System.out.println("The server will then execute a CallControlCall.offHook() service and generate the carrier tone for the other terminal.");
								}
							catch (InterruptedException exc) {}
							}
						};
					new Thread(runnable).start();
					break;
			}
		}
	}
}
	
