/*
 * $Source: /cvs/sources/com/odisei/sdk/samples/hold/HoldMain.java,v $
 * $Revision: 1.6 $
 * $Date: 1999/09/30 15:29:07 $
 * $Author: kris $
 *
 * Copyright (c) 1999 by 8x8, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of 8x8. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with 8x8.
 *
 */

import java.io.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;

/**
 This code illustrates usage of hold() / unhold() services.
 receives and manages CallControlCall events.
 This scenario execute :
 1- AddToAddress() service to establish a connection between phones 101 and 102.
 2- Hold() service on TerminalConnection 'established'.
 3- Unhold() service to retrieve the connection.
*/

public class HoldMain {
	private static BufferedInputStream buffered = new BufferedInputStream(System.in);
	private static Provider myProvider ;
	private static boolean exit = true;
	final static int NO_ACTION = 0;
	final static int ACTION = 1;
	final static int SHUTDOWN = 2;


	public static final void main(String[] args) {

		/**
		Create a Provider by first obtaining the 8x8 implementation of
		JTAPI and then the specific Provider implementation indicated by the Provider String.
		The CTI Server host address is specified in that string.
		*/

		String ctiServerHost = null;
		if (args == null || args.length < 1)
			throw new RuntimeException("Need url server for connecting provider");

		ctiServerHost = args[0];

		try {
			JtapiPeer peer = JtapiPeerFactory.getJtapiPeer("com.odisei.server.x.engine.EngineJtapiPeer");
			myProvider = peer.getProvider("cce;com.odisei.telephony.host=" + ctiServerHost +  ";login=admin;passwd=admin");
			}
		catch (Exception e) {
			throw new RuntimeException("Can't get Provider "+ e.toString());
			}

		/*
   		Get a Terminal instance and add a CallObserver to it
	   	*/
		try {
			Terminal terminal101 = myProvider.getTerminal("101");
			terminal101.addCallObserver(new HoldObserver(terminal101));
			}
		catch (Exception e) {
			throw new RuntimeException("Can't get terminal");
			}

		System.out.println("Please pick-up the phone 101.");
	}

	/**
	Prompt the user to take an action on the keyboard
	*/
	static synchronized int executeRequest(String request) {
		exit = true;
 		System.out.println(request + " or q + <enter> to stop the application");
   		System.out.print("> ");
		int b = 0;
	  	while (exit && b == 0)  {
	   		try {
	   			Thread.sleep(500);
				if (!exit) return NO_ACTION;
					   			
	   			if(buffered.available() > 0) {
	   				String s = new BufferedReader(new InputStreamReader(buffered)).readLine();
	   				if (s.equalsIgnoreCase("q")) 
	   					return SHUTDOWN;
	   				else 
	   					return ACTION;	
   				}		
   			}
	   		catch (Exception e ) {
	    		return NO_ACTION;
   			}
  		}
	  	return ACTION;
	}

	static void stopRequest() {
		exit = false;
	} 

	/**
	Shut down Provider and quit program.
	Instructs the Provider to shut down itself and perform all necessary cleanup.
	Note that this will not affect the CTI server, it is just a 'log-out' by the client.
	 */
	static void shutdown() {
		myProvider.shutdown();
		System.exit(0);
	} 	
}
	
