/*
 * $Source: /cvs/sources/com/odisei/sdk/samples/hold/HoldObserver.java,v $
 * $Revision: 1.5 $
 * $Date: 1999/09/30 15:29:08 $
 * $Author: kris $
 *
 * Copyright (c) 1999 by 8x8, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of 8x8. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with 8x8.
 *
 */

import java.io.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;


/**
 This class observes and displays CallControl events.
 Additional information about executed JTAPI services is displayed when the
 service behaviour is not defined unambiguosly by Suns JTAPI documentation.
 */
 public class HoldObserver implements CallControlCallObserver {
	private Terminal myTerminal = null;
	private Thread thread = null;
	
	public HoldObserver(Terminal terminal) {
		myTerminal = terminal;
	}

	/** Implementation of callChangedEvent() */
	public synchronized void callChangedEvent(CallEv[] eventList) {
 		CallCtlConnEv connEvent = null;
 		CallCtlTermConnEv termConnEvent = null;
 		String msg = null;

 		for (int i = 0; i < eventList.length; i++) {

 			if (eventList[i] instanceof CallCtlConnEv) {
 				connEvent = (CallCtlConnEv)eventList[i];
 				Address address = connEvent.getConnection().getAddress();
 				msg = "Connection to address : " + address.getName() + " is ";
 			}
 			else if (eventList[i] instanceof CallCtlTermConnEv) {
 				termConnEvent = (CallCtlTermConnEv)eventList[i];
 				Terminal terminal = termConnEvent.getTerminalConnection().getTerminal();
 				msg = "TerminalConnection : " + terminal.getName() + " is ";
			}
 			else
 				continue;

 			switch (eventList[i].getID()) {
	 			case CallCtlConnFailedEv.ID :
	 				System.out.println(msg + "Failed");
	 				break;

	 			case CallCtlConnAlertingEv.ID :
	 				System.out.println(msg + "Alerting");
	 				break;

				/*
				Scan TerminalConnection of all (Initiated) Connections:
	            Unhold() service will be invoked on TerminalConnections which are in Held state,
    	        else addToAddress() is invoked on the Connection.
				 */
 				case CallCtlConnInitiatedEv.ID :
 					System.out.println(msg + "Initiated");
					final CallControlConnection connection = (CallControlConnection)connEvent.getConnection();
 					boolean isHeld = false;
 					if (thread != null)	thread.interrupt();  
		
					TerminalConnection[] term = myTerminal.getTerminalConnections();
					for (int z = 0; term != null && z < term.length; z++) {
						if (((CallControlTerminalConnection)term[z]).getCallControlState() == CallControlTerminalConnection.HELD) {
							isHeld = true;
							final CallControlTerminalConnection terminalConnection = (CallControlTerminalConnection) term[z];
							thread = new Thread() {
									public void run() {
										switch(HoldMain.executeRequest("Hit <enter> to execute Unhold(102) service")) {
											case HoldMain.SHUTDOWN :
		 										HoldMain.shutdown();									
												break;
											
											case HoldMain.ACTION :										
												try {
													terminalConnection.unhold();
													System.out.println("---------------------------------");
													System.out.println("Unhold() service destroyed connection in Initiated state.");
													System.out.println("---------------------------------");
												}
												catch(Exception e) {
													System.out.println("Can't invoke unhold() service.");
												}
												break;

											case HoldMain.NO_ACTION :
		 										break;
										}
									}
								};
							thread.start();	
							break;
						}	
					}	
						
					if (!isHeld) {
					 	thread = new Thread() {
								public void run(){
									switch(HoldMain.executeRequest("Hit <enter> to execute addToAddress(102) service")) {
										case HoldMain.SHUTDOWN :
		 									HoldMain.shutdown();									
											break;
											
										case HoldMain.ACTION :
				 							try {	
						   	               // It is a good attitude not to block JTAPI event handlers
												connection.addToAddress("102");
		 									}
											catch (Exception e) {
												System.out.println("can't invoke addToAddress" + e.toString());
											}
											break;
											
										case HoldMain.NO_ACTION :
											break;
									}
								}
								public void interrupt() {
									HoldMain.stopRequest();
									super.interrupt();	
								}
							};
 						thread.start();
					}
					break;

 				case CallCtlConnDialingEv.ID :
 					System.out.println(msg + "Dialing");
 					break;

 				case  CallCtlConnDisconnectedEv.ID :
 					System.out.println(msg + "Disconnected");
 					break;

 				case CallCtlConnEstablishedEv.ID :
 					System.out.println(msg + "Established");
 					break;

 				case CallCtlTermConnDroppedEv.ID :
 					System.out.println(msg + "Dropped");
 					break;

 				case CallCtlTermConnRingingEv.ID :
	 				System.out.println(msg + "Alerting, please pick-up this phone.");
	 				break;

				case CallCtlTermConnTalkingEv.ID :
					System.out.println(msg + "Talking");
					Terminal terminal = termConnEvent.getTerminalConnection().getTerminal();
					if (myTerminal.equals(terminal)) break;
					
					if (thread != null)	thread.interrupt();
					final Call call = termConnEvent.getCall();

					thread	= new Thread() {
							public void run() {
								switch(HoldMain.executeRequest("Hit <enter> to execute hold service")) {
									case HoldMain.SHUTDOWN :
	 									HoldMain.shutdown();									
										break;
									
									case HoldMain.ACTION :
										// Execute hold once connection is established:
										// Check that (Terminal)Connections for 101 & 102 are in Establhished-Talking state.

										TerminalConnection[] terms = myTerminal.getTerminalConnections();
										for (int z = 0; terms != null && z < terms.length; z++) {
									
											CallControlTerminalConnection termconn = (CallControlTerminalConnection)terms[z];
											CallControlConnection conn = (CallControlConnection)terms[z].getConnection(); 	
										
											if (termconn.getCallControlState() == CallControlTerminalConnection.TALKING &&
								 				conn.getCallControlState() == CallControlConnection.ESTABLISHED &&
								 				call.equals(terms[z].getConnection().getCall()) == true) {
							 				
								 				try {
													termconn.hold();
													}
												catch (Exception exc) {
													System.out.println("can't execute hold() service" + exc.toString());
													}
												break;
											}
								 		}
									break;
 									
 									case HoldMain.NO_ACTION :
 										break;
								}
							}
	 						public void interrupt() {
	 							HoldMain.stopRequest();
	 							super.interrupt();	
							}
						};
 					thread.start();	
					break;

				case CallCtlTermConnHeldEv.ID :
					System.out.println("---------------------------------------------");
					System.out.println(msg + "Held");
					System.out.println("When a Connection is put on hold, the server executes an offHook() & generates a carrier tone for the corresponding Terminal.");
					System.out.println("Generally, the server invokes an offHook() service & generates a carrier tone");
   	        		System.out.println("whenever a Terminal enters Hookswitch Off state without having (any more) TerminalConnections");
					System.out.println("in Talking state.");
					System.out.println("---------------------------------------------");
					break;
			}
		}
 	}
 }
