/*
 * $Source: /cvs/sources/com/odisei/sdk/samples/transfer/TransferMain.java,v $
 * $Revision: 1.7 $
 * $Date: 1999/09/30 16:02:03 $
 * $Author: kris $
 *
 * Copyright (c) 1999 by 8x8, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of 8x8. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with 8x8.
 *
 */

import java.io.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;
import javax.telephony.phone.*;
import javax.telephony.phone.events.*;

/**
This code illustrates a single step transfer. In this scenario, we:
 1- Observe phone events, waiting for a hookswitch off event.
 2- Use the connect() service to establish a connection between phones 101 and 102.
 3- Transfer the connection from phone 101 to 103.
*/
public class TransferMain {

	private static BufferedInputStream buffered = new BufferedInputStream(System.in);
	private static Provider myProvider ;
	private static boolean exit = true;
	final static int NO_ACTION = 0;
	final static int ACTION = 1;
	final static int SHUTDOWN = 2;

	public static final void main(String[] args) {

		/*
		Create a Provider by first obtaining the 8x8 implementation of
		JTAPI and then the specific Provider implementation indicated by the Provider String.
		The CTI Server host address is specified in that string.
		*/

		String ctiServerHost = null;
		if (args == null || args.length < 1)
			throw new RuntimeException("Need url server for connecting provider");

		ctiServerHost = args[0];

		try {
			JtapiPeer peer = JtapiPeerFactory.getJtapiPeer("com.odisei.server.x.engine.EngineJtapiPeer");
			myProvider = peer.getProvider("cce;com.odisei.telephony.host=" + ctiServerHost +  ";login=admin;passwd=admin");
		}
		catch (Exception e) {
			throw new RuntimeException("Can't get Provider "+ e.toString());
		}


		/* Get a Terminal and add a CallObserver to it */
		try {
			final Provider provider = myProvider;
			final Terminal terminal101 = provider.getTerminal("101");
			terminal101.addCallObserver(new TransferObserver(terminal101));

			/*
			Inner class Terminal event observer:
			We invoke connect() in a separate event handler thread (in good JTAPI manner)
			when a Hookswitch Off is detected
			*/
			terminal101.addObserver(new PhoneTerminalObserver() {

					public void terminalChangedEvent(TermEv[] eventList) {
						for (int i = 0; i < eventList.length; i++) {

							if (eventList[i].getID() != HookswitchStateEv.ID) continue;

							if (((HookswitchStateEv)eventList[i]).getHookSwitchState() == PhoneHookswitch.ON_HOOK) {
								System.out.println("Phone " + terminal101.getName() + " is hookswitch on state ");
								stopRequest();
								new Thread() {
										public void run() {						
											switch(executeRequest("Hit <enter> to execute connect(102) service")) {
									
												case SHUTDOWN :
			 										shutdown();									
													break;
											
												case ACTION :
					 								try {	
									                // It is a good attitude not to block JTAPI event handlers
													provider.createCall().connect(terminal101, terminal101.getAddresses()[0], "102");												
				 									}
													catch (Exception e) {
														System.out.println("Error : " + e.toString());
													}
													break;
											
												case NO_ACTION :
													break;
											}
										}
									}.start();
							}
						}
					}
				}
			);
		}
		catch (Exception e) {
			throw new RuntimeException("Can't get terminal");
		}
	}

	/**
	Prompt the user to take an action on the keyboard
	*/
	static synchronized int executeRequest(String request) {
		exit = true;
 		System.out.println(request + " or q + <enter> to stop the application");
   		System.out.print("> ");
		int b = 0;
	  	while (exit && b == 0)  {
	   		try {
	   			Thread.sleep(500);
				if (!exit) return NO_ACTION;
					   			
	   			if(buffered.available() > 0) {
	   				String s = new BufferedReader(new InputStreamReader(buffered)).readLine();
	   				if (s.equalsIgnoreCase("q")) 
	   					return SHUTDOWN;
	   				else 
	   					return ACTION;	
   				}		
   			}
	   		catch (Exception e ) {
	    		return NO_ACTION;
   			}
  		}
	  	return ACTION;
	}

	static void stopRequest() {
		exit = false;
	} 

	/**
	Shut down Provider and quit program.
	Instructs the Provider to shut down itself and perform all necessary cleanup.
	Note that this will not affect the CTI server, it is just a 'log-out' by the client.
	*/
	static void shutdown() {
		myProvider.shutdown();
		System.exit(0);
	}
}
