/*
 * $Source: /cvs/sources/com/odisei/sdk/samples/transfer/TransferObserver.java,v $
 * $Revision: 1.7 $
 * $Date: 1999/09/30 16:02:03 $
 * $Author: kris $
 *
 * Copyright (c) 1999 by 8x8, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of 8x8. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with 8x8.
 *
 */

import java.io.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;


/**
This class observes and displays CallControl events.
Additional information about executed JTAPI services is displayed when the
service behaviour is not defined unambiguosly by Suns JTAPI documentation.
*/
public class TransferObserver implements CallControlCallObserver {
	private Terminal myTerminal = null;
	private BufferedReader buffered;

	public TransferObserver(Terminal terminal) {
		myTerminal = terminal;
	}  

	/** Implementation of callChangedEvent */
	public void callChangedEvent(CallEv[] eventList) {
 		CallCtlConnEv connEvent = null;
 		CallCtlTermConnEv termConnEvent = null;
 		String msg = null;
 		
 		for (int i = 0; i < eventList.length; i++) {
 		
 			if (eventList[i] instanceof CallCtlConnEv) {
 				connEvent = (CallCtlConnEv)eventList[i];
 				Address address = connEvent.getConnection().getAddress();
 				msg = "Connection to address : " + address.getName() + " is ";
 				}
 		
 			else if (eventList[i] instanceof CallCtlTermConnEv) {
 				termConnEvent = (CallCtlTermConnEv)eventList[i];
 				Terminal terminal = termConnEvent.getTerminalConnection().getTerminal(); 
 				msg = "TerminalConnection : " + terminal.getName() + " is ";   				
 				}
 			
 			else continue;
 					
 			switch (eventList[i].getID()) {
	 			case CallCtlConnFailedEv.ID :
	 				System.out.println(msg + "Failed");
	 				break;
	 			
	 			case CallCtlConnAlertingEv.ID :
	 				System.out.println(msg + "Alerting");
	 				break;

 				case CallCtlConnInitiatedEv.ID :
 					System.out.println(msg + "Initiated");
 					
	 				
 				case CallCtlConnDialingEv.ID :
 					System.out.println(msg + "Dialing");
 					break;
 					
 				case  CallCtlConnDisconnectedEv.ID :
 					System.out.println(msg + "Disconnected");
 					break;
 					
 				case CallCtlConnEstablishedEv.ID :
 					System.out.println(msg + "Established");
 					break;
 					
 				case CallCtlTermConnDroppedEv.ID :
 					System.out.println(msg + "Dropped");
 					break;
 					
 				case CallCtlTermConnRingingEv.ID :	 		 	
	 				System.out.println(msg + "Alerting, please pick-up this phone.");
	 				break;
	 				
				case CallCtlTermConnTalkingEv.ID :
					System.out.println(msg + "Talking");
					Terminal terminal = termConnEvent.getTerminalConnection().getTerminal();
					if (myTerminal.equals(terminal)) break;
						
					TransferMain.stopRequest();
					final CallControlCall call = (CallControlCall)termConnEvent.getCall();
					new Thread() {
							public void run(){
						
								switch(TransferMain.executeRequest("Press return to execute transfer service (call 101-102 --> 102-103).")) {
									case TransferMain.SHUTDOWN :
	 									TransferMain.shutdown();									
										break;
									
									// Get the TerminalConnection to be transferred and execute transfer() service.
									case TransferMain.ACTION :
										TerminalConnection[] terms = myTerminal.getTerminalConnections();
										for (int z = 0; terms != null && z < terms.length; z++) {
											CallControlTerminalConnection term = (CallControlTerminalConnection)terms[z];
											CallControlConnection conn = (CallControlConnection)terms[z].getConnection();
										  
											if (term.getCallControlState() == CallControlTerminalConnection.TALKING &&
									 			conn.getCallControlState() == CallControlConnection.ESTABLISHED &&
									 			call.equals(terms[z].getConnection().getCall()) == true) {
									 			try {
													call.setTransferController(terms[z]);
													call.transfer("103");
												}
												catch (Exception exc) {
													System.out.println("can't execute transfert service" + exc.toString());
												}
		 										break;	
											}	
 										}	
										break;
									 									
 									case TransferMain.NO_ACTION :
 										break;
								}
 							}
 						}.start();
 					break;
			}
		}
 	}
 }
